PR.DEBUG = false
PR.debugCommands = {}

--[[
local function alphanumsort2(o)
	local function padnum(d) return ("%03d%s"):format(#d, d) end

	table.sort(o, function(a, b)
		return tostring(a):gsub("%d+", padnum) < tostring(b):gsub("%d+", padnum)
	end)
	return o
end

local function sortfunc(a, b)
	local function pad(d) return ("%09d%s"):format(#d, d) end

	return tostring(a):gsub("%d+", pad) < tostring(b):gsub("%d+", pad)
end
]]

local function debugTarget()
    local objectid, nodeid
    nodeid, objectid = radar.GetRadarSelectionID()
    print("DEBUG: target " .. tostring(nodeid) .. "/" .. tostring(objectid))
end

local function debugTargetToggle()
    if PR.DEBUG_TARGET then
        PR.DEBUG_TARGET = nil
        UnregisterEvent(debugTarget, "TARGET_CHANGED")
    else
        PR.DEBUG_TARGET = true
        RegisterEvent(debugTarget, "TARGET_CHANGED")
        print("DEBUG: WARNING, do not run /pr_debug_target at the same time as the auto-scan, the high volume of targets will fill the error.log and then crash the game")
    end
end

local function debugHelp()
    print("Debug functions:")
    for f in pairs(PR.debugCommands) do
        print("  /" .. f)
    end
end

local function debugToggle()
    PR.DEBUG = not PR.DEBUG
    PR.DebugPrint("Debug is " .. tostring(PR.DEBUG))
end

PR.debugCommands = {
    ["pr_debug"] = debugToggle,
    ["pr_debug_help"] = debugHelp,
    ["pr_debug_target"] = debugTargetToggle,
}

for command, callback in pairs(PR.debugCommands) do
    RegisterUserCommand(command, callback)
end

function PR.FindOver100()
    for systemId = 1, PR.SystemOwnership.n do
        for sectorId = systemId * 256 - 255, systemId * 256 do
            local sectorData = PR.LoadSectorData(sectorId)
            if sectorData and type(sectorData) == "table" and next(sectorData) ~= nil then
                if sectorData.RoidList then
                    for roidId, roidData in pairs(sectorData.RoidList) do
                        local total = 0.0
                        for i = 1, PR.Ore.n do
                            total = total + (roidData[PR.Ore[i].shortname] or 0)
                        end
                        if total >= 100.2 then
                            PR.DebugPrint("roid " .. roidId .. " in sector " .. sectorId ..
                                " has total ore%: " .. total)
                        end
                    end
                end
                --PR.SaveSectorData(sectorId, sectorData)
            end
        end
    end
end

RegisterUserCommand("FindOver100", PR.FindOver100)

function PR.CheckScanner()
    --[[ 
        -- itemids are allegedly variable but are static in all my tests)
        -- itemType is static, can be converted to a name with InvManager.InventoryItemTypeToName()
        GetInventoryItemType(19) -> 859300209
        GetInventoryItemInfo(19) ->
            name:Mineral Scanner
            icon:images/icon_addon_scanner.png
            qty:1
            mass:0.10000000149012
            sdesc:Scans asteroid mineral contents
            desc1:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            desc2:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            containerId:142
            class:2
            subtype:0
        GetInventoryItemInfo(9) ->
            name:Free Mineral Scanner
            icon:images/icon_addon_scanner.png
            qty:1
            mass:0.10000000149012
            sdesc:Scans asteroid mineral contents
            desc1:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            desc2:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            containerId:1
            class:2
            subtype:0
        GetInventoryItemType(1597) -> 2137992586
        GetInventoryItemInfo(1597) ->
            name:Scanner
            icon:images/icon_addon_scanner.png
            qty:1
            mass:0.10000000149012
            sdesc:Scanner to scan targets
            desc1:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            desc2:Range: 500|Energy: 0/scan|Grid Usage: 4|Mass: 100 kg|Volume: 1 cu
            containerId:1585
            class:2
            subtype:0
        GetInventoryItemType(1816) -> 3739338437
        GetInventoryItemInfo(1816) ->
            name:Advanced Mineral Scanner
            icon:images/icon_addon_scanner.png
            qty:1
            mass:0.10000000149012
            sdesc:Scans asteroid mineral contents
            desc1:Range: 750|Energy: 0/scan|Grid Usage: 1|Mass: 100 kg|Volume: 1 cu
            desc2:Range: 750|Energy: 0/scan|Grid Usage: 1|Mass: 100 kg|Volume: 1 cu
            containerId:1812
            class:2
            subtype:0
        GetInventoryItemType(1898) -> 3522346340
        GetInventoryItemInfo(1898) ->
            name:Cargo Scanner
            icon:images/icon_addon_scanner.png
            qty:1
            mass:0.10000000149012
            sdesc:Scans cargo hold of targeted ship
            desc1:Range: 2000|Grid Usage: 1|Mass: 100 kg|Volume: 1 cu
            desc2:Range: 2000|Grid Usage: 1|Mass: 100 kg|Volume: 1 cu
            containerId:1901
            class:2
            subtype:0
    ]]

    for _, itemid in ipairs(GetShipInventory(GetActiveShipID()).addons) do
        local item_name = GetInventoryItemName(itemid)
        local item_type = GetInventoryItemType(itemid)
        local item_class = InvManager.GetInventoryItemParameter(item_type, "Class") or ""
        local range = tonumber(InvManager.GetInventoryItemParameter(item_type, "Range") or 0)
        if item_class == "Scanner" then
            PR.DebugPrint("name: " .. item_name .. ", type: " .. item_type .. ", class: " .. item_class .. ", id: " .. itemid)
            PR.DebugPrint("Range is " .. range)
            local icon, name, qty, mass, sdesc, desc1, desc2, containerId, class, subtype = GetInventoryItemInfo(itemid)
            PR.DebugPrint("name:" .. name .. ",icon:" .. icon .. ",qty:" .. qty .. ",mass:" .. mass .. ",sdesc:" .. sdesc ..
                ",desc1:" .. desc1 .. ",desc2:" .. desc2 .. ",containerId:" .. containerId .. ",class:" .. class .. ",subtype:" .. subtype)
        elseif item_class == "MiningGun" then
            PR.DebugPrint("name: " .. item_name .. ", type: " .. item_type .. ", class: " .. item_class .. ", id: " .. itemid)
            local icon, name, qty, mass, sdesc, desc1, desc2, containerId, class, subtype = GetInventoryItemInfo(itemid)
            PR.DebugPrint("name:" .. name .. ",icon:" .. icon .. ",qty:" .. qty .. ",mass:" .. mass .. ",sdesc:" .. sdesc ..
                ",desc1:" .. desc1 .. ",desc2:" .. desc2 .. ",containerId:" .. containerId .. ",class:" .. class .. ",subtype:" .. subtype)
        end
    end
end

RegisterUserCommand("checkscanner", PR.CheckScanner)

function PR.DebugPrint(text)
    text = tostring(text)
    print(PR.Color.Red .. "DEBUG: " .. text .. PR.Color.TERMINATOR)
    log_chat("DEBUG: " .. text)
end

function PR.TestEvent(event, ...)
    local outstring = event
    local arg = { ... }
    for i, v in ipairs(arg) do
        outstring = outstring .. ", arg[" .. i .. "] = " .. tostring(v)
    end
    PR.DebugPrint(outstring)
end

--[[
RegisterEvent(PR.TestEvent, "PLUGINS_LOADED")
-- no args
RegisterEvent(PR.TestEvent, "START")
-- no args
RegisterEvent(PR.TestEvent, "LOGIN_SUCCESSFUL") -- triggers after login, when at charselect
-- no args
RegisterEvent(PR.TestEvent, "PLAYER_ENTERED_GAME") -- after loading in with a char
-- no args
RegisterEvent(PR.TestEvent, "SECTOR_CHANGED") -- triggers when you load into the game
-- arg1: sectorId
RegisterEvent(PR.TestEvent, "PLAYER_UPDATE") -- also on first load in, on every dock/undock. basically a reliable location change (and more)
-- arg1: playerId
RegisterEvent(PR.TestEvent, "SECTOR_LOADING") -- doesn't trigger
RegisterEvent(PR.TestEvent, "SECTOR_LOADED") -- triggers when everything is fully loaded
-- no args
RegisterEvent(PR.TestEvent, "SHIP_CHANGED") -- when ship gets changed, including on first load in. arg1 is itemid
-- arg1: itemId
RegisterEvent(PR.TestEvent, "SHIP_UPDATED") -- always in station and probably cap ships. arg1 is itemid
-- arg1: itemId
RegisterEvent(PR.TestEvent, "SHIP_SPAWNED") -- when ship gets spawned in space, after every undock or sector load
-- no args
RegisterEvent(PR.TestEvent, "ENTERING_STATION")
-- no args
RegisterEvent(PR.TestEvent, "ENTERED_STATION")
-- no args
RegisterEvent(PR.TestEvent, "INVENTORY_UPDATE") -- always in station and probably cap ships, not for ship cargo operations in space. arg1 is itemid
-- arg1: itemId
RegisterEvent(PR.TestEvent, "PLAYER_LOGGED_OUT")
-- no args
RegisterEvent(PR.TestEvent, "QUIT")
RegisterEvent(PR.TestEvent, "PLAYER_DIED")
RegisterEvent(PR.TestEvent, "TARGET_SCANNED")
-- arg1 is scaninfo, arg2 is node, arg3 is objectid
RegisterEvent(PR.TestEvent, "TARGET_CHANGED")
-- no args
RegisterEvent(PR.TestEvent, "OBJECT_INFO_UPDATED") -- not sure when it triggers
-- arg1: name, arg2: name, arg3: 0 (for asteroid)
RegisterEvent(PR.TestEvent, "STORM_STARTED") -- triggers when entering sector with storm and when storm starts
-- no args
RegisterEvent(PR.TestEvent, "STORM_STOPPED") -- triggers when entering sector without storm and when storm stops
-- no args
]]

--[[ RegisterEvent(PR.TestEvent, "CHAT_MSG_CHANNEL")
RegisterEvent(PR.TestEvent, "CHAT_MSG_CHANNEL_ACTIVE")
RegisterEvent(PR.TestEvent, "CHAT_MSG_DISABLED")
RegisterEvent(PR.TestEvent, "CHAT_MSG_ERROR")
RegisterEvent(PR.TestEvent, "CHAT_MSG_GLOBAL_SERVER")
RegisterEvent(PR.TestEvent, "CHAT_MSG_MISSION")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SECTORD")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SECTORD_MISSION")
-- HUD:PrintSecondaryMsg triggers this
RegisterEvent(PR.TestEvent, "CHAT_MSG_GROUP")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SERVER")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SERVER_CHANNEL")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SERVER_CHANNEL_ACTIVE")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SERVER_SECTOR")
RegisterEvent(PR.TestEvent, "CHAT_MSG_SYSTEM")
]]

--[[ RegisterEvent(PR.TestEvent, "HUD_RELOADED")
RegisterEvent(PR.TestEvent, "HUD_HIDE")
RegisterEvent(PR.TestEvent, "HUD_UPDATE")
RegisterEvent(PR.TestEvent, "HUD_SHOW")
RegisterEvent(PR.TestEvent, "HUD_MODE_TOGGLE")
RegisterEvent(PR.TestEvent, "HUD_TOGGLE")
RegisterEvent(PR.TestEvent, "HUD_INVENTORY_TOGGLE") ]]

--[[ -- arg1 is itemid, arg2 is always 0 it seems, arg3 is probably progressbarId and arg4 the progress
RegisterEvent(PR.TestEvent, "WEAPON_PROGRESS_START") -- when weapons/mining beams with progress bars get activated
-- arg1 is itemid
RegisterEvent(PR.TestEvent, "WEAPON_PROGRESS_STOP") -- when progress stops
 ]]

--[[
 [05/18/22 08:10:56] DEBUG: PLUGINS_LOADED
 [05/18/22 08:10:56] DEBUG: START
 [05/18/22 08:10:57] Welcome to Vendetta Online.
 [05/18/22 08:10:57] DEBUG: LOGIN_SUCCESSFUL
 [05/18/22 08:11:14] DEBUG: SECTOR_CHANGED
 [05/18/22 08:11:14] You are entering Helios B-7
 [05/18/22 08:11:15] DEBUG: PLAYER_UPDATE
 [05/18/22 08:11:16] DEBUG: SECTOR_LOADED
 [05/18/22 08:11:16] You are now in Helios Outpost (Serco Mining).
 [05/18/22 08:11:16] DEBUG: PR.Init
 [05/18/22 08:12:02] DEBUG: PLAYER_LOGGED_OUT
 [05/18/22 08:12:03] DEBUG: QUIT
]]


-- toggle targetless TARGET_CHANGED event handlers
function PR.TargetlessOverrideToggle()
    if not PR.TargetlessOverrideStatus then
        UnregisterEvent(targetless.update, "TARGET_CHANGED")
        UnregisterEvent(targetless.roidtype, "TARGET_CHANGED")
        PR.TargetlessOverrideStatus = true
        print("Prospector Targetless Override activated")
    else
        RegisterEvent(targetless.update, "TARGET_CHANGED")
        RegisterEvent(targetless.roidtype, "TARGET_CHANGED")
        PR.TargetlessOverrideStatus = false
        print("Prospector Targetless Override deactivated")
    end
end

RegisterUserCommand("prTargetlessOverride", PR.TargetlessOverrideToggle)

--[[
InventoryItem class has the following known methods:
:GetName()
:SetName() -- doesn't seem to work
:GetMass()
:SetMass() -- works
:GetIcon()
:SetIcon()
:GetDesc() -- textual description
:SetDesc() -- doesn't seem to work
:GetLongDesc() -- item stats
:SetLongDesc()
:GetExtendedDesc() -- same as above, on ships also LongDesc
:GetVolume()
:SetVolume()
:GetDamage() -- returns nil, might need an argument
:SetDamage()
:Destroy() -- dunno what this does
:Use() -- needs argument
]]
function PR.InvItemTest()
    PR.InventoryItem = InvManager.GetInventoryItemPrototype(2836556204) -- HD Beam
end

-- HD Beams: 2836556204
-- Improved: 4213090619
-- Standard: 2519511976
--[[
InvManager.GetInventoryItemParameter(2836556204, "Range") or 0)
known valid parameters for addons:
    Name
    Range -- in meters
    Mass -- in tons
    Volume -- in cargo units
    Class -- not all items have one, but scanners + mining beams do
    Icon
    Desc -- textual description
    LongDesc -- item stats
]]
--[[
    PR.InventoryItem = InvManager.GetInventoryItemPrototype(3675049252)
-- Behemoth Heavy Miner MkII: 3675049252
InvManager.GetInventoryItemParameter(3675049252, "Name")
known valid parameters for ships:
    Name
    Volume -- in cargo units
    Cargo -- total cargo space
    Length -- in meters
    Icon
    Desc -- the short description on the buy screen: "Behemoth Heavy Miner MkII - Requires Basic Miner III"
    LongDesc -- the optional long description at the bottom: "The Behemoth is the heavyweight hauler of sub-Capital scale vessels. Excellent at transporting large scale goods, not great at everything else. This mining variant can be configured with one turret."

 ]]

